==> pvfs-1.3.2.tgz: pvfs-1.3.2/lib/pvfs_truncate.c <==
/*
	pvfs_truncate.c copyright (c) 1998 Clemson University
   all rights reserved.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Walt Ligon walt@eng.clemson.edu

/*/

/*
 * $Log: pvfs_truncate.c,v $
 * Revision 1.1  1998/07/23 21:54:06  rbross
 * Initial revision
 *
 *
/*/

static char pvfs_truncate_c[] = "$Header: /projects/pvfs/pvfs-working/lib/RCS/pvfs_truncate.c,v 1.1 1998/07/23 21:54:06 rbross Exp $";

#include <lib.h>
#include <meta.h>

extern int errno;

int pvfs_truncate(const char *path, size_t length)
{
	int i, s;
	mreq request;
	mack ack;
	char *d_p;
	dmeta dir;

	if (!path) {
		errno = EFAULT;
		return(-1);
	}

	/* check to see if file is a pvfs file */
	if ((i = check_pvfs(path)) != 1) {
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		return(syscall(SYS_truncate, path, length));
	}
	if (get_dmeta(path, &dir) < 0) {
		PERROR("Error getting file");
		return(-1);
	}

	/* Prepare request for file system  */
	request.uid = getuid();
	request.gid = getgid();
	request.type = MGR_TRUNCATE;
	request.dsize = strlen(path) + 1;
	request.req.truncate.fname = 0;
	request.req.truncate.length = length;

	if ((d_p = malloc(request.dsize)) == NULL) {
		PERROR("malloc");
		return(-1);
	}
	strcpy(d_p, path);
	d_p[request.dsize] = '\0';


	/* Send request to mgr */	
	if ((s = send_mreq_dmeta(&dir, &request, d_p, &ack)) < 0) {
		PERROR("send_mreq");
		free(d_p);
		return(-1);
	}
	free(d_p);
	return(0);
}

==> pvfs-1.4.2.tgz: pvfs-1.4.2/lib/pvfs_truncate.c <==
/*
	pvfs_truncate.c copyright (c) 1998 Clemson University
   all rights reserved.

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as published
	by the Free Software Foundation; either version 2 of the License, or
	any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of 
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
	General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

	Contact:  Rob Ross  	rbross@parl.eng.clemson.edu
				 Walt Ligon walt@eng.clemson.edu

 */

static char pvfs_truncate_c[] = "$Header: /projects/cvsroot/pvfs/lib/pvfs_truncate.c,v 1.3 1999/10/15 22:05:57 rbross Exp $";

#include <lib.h>
#include <meta.h>

extern int errno;
extern int pvfs_checks_disabled;

int pvfs_truncate(const char *path, size_t length)
{
	int i, s;
	mreq request;
	mack ack;
#ifdef __USE_NFS__
	char *d_p;
	dmeta dir;
#else
	struct sockaddr *saddr;
	char *fn;
	int64_t fs_ino;
#endif

	if (!path) {
		errno = EFAULT;
		return(-1);
	}

	if (pvfs_checks_disabled) return(syscall(SYS_truncate, path, length));

	/* check to see if file is a pvfs file */
#ifdef __USE_NFS__
	if ((i = check_pvfs(path)) != 1) {
#else
	if ((i = pvfs_detect(path, &fn, &saddr, &fs_ino)) != 1) {
#endif
		if (i < 0) {
			PERROR("Error finding file");
			return(-1);
		}	
		if (i == 2) {
			errno = EISDIR;
			return(-1);
		}
		return(syscall(SYS_truncate, path, length));
	}
#ifdef __USE_NFS__
	if (get_dmeta(path, &dir) < 0) {
		PERROR("Error getting file");
		return(-1);
	}
#endif

	/* Prepare request for file system  */
	request.uid = getuid();
	request.gid = getgid();
	request.type = MGR_TRUNCATE;
#ifdef __USE_NFS__
	request.dsize = strlen(path) + 1; /* THIS WON'T ALWAYS WORK EVEN WITH NFS */
#else
	request.dsize = strlen(fn);
#endif
	request.req.truncate.fname = 0;
	request.req.truncate.length = length;

#ifdef __USE_NFS__
	if ((d_p = malloc(request.dsize)) == NULL) {
		PERROR("malloc");
		return(-1);
	}
	strcpy(d_p, path);
	d_p[request.dsize] = '\0';
#endif


	/* Send request to mgr */	
#ifdef __USE_NFS__
	if ((s = send_mreq_dmeta(&dir, &request, d_p, &ack)) < 0) {
#else
	if ((s = send_mreq_saddr(saddr, &request, fn, &ack)) < 0) {
#endif
		PERROR("send_mreq");
#ifdef __USE_NFS__
		free(d_p);
#endif
		return(-1);
	}
#ifdef __USE_NFS__
	free(d_p);
#endif
	return(0);
}
